using System.ComponentModel;
using System.Windows.Input;

namespace TestClient;

/// <summary>
/// ǰ ߰  
/// </summary>
public class AddPartViewModel : INotifyPropertyChanged
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Event
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region Ӽ  - PropertyChanged

    /// <summary>
    /// Ӽ 
    /// </summary>
    public event PropertyChangedEventHandler PropertyChanged;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// ǰ ID
    /// </summary>
    private string partID;

    /// <summary>
    /// ǰ
    /// </summary>
    private string partName;

    /// <summary>
    ///  Ʈ
    /// </summary>
    private string supplierList;

    /// <summary>
    /// ǰ Ÿ
    /// </summary>
    private string partType;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Property
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region ǰ ID - PartID

    /// <summary>
    /// ǰ ID
    /// </summary>
    public string PartID
    {
        get => this.partID;
        set
        {
            if(this.partID == value)
            {
                return;
            }

            this.partID = value;

            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(PartID)));
        }
    }

    #endregion
    #region ǰ - PartName

    /// <summary>
    /// ǰ
    /// </summary>
    public string PartName
    {
        get => this.partName;
        set
        {
            if(this.partName == value)
            {
                return;
            }

            this.partName = value;

            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(PartName)));
        }
    }

    #endregion
    #region  Ʈ - SupplierList

    /// <summary>
    ///  Ʈ
    /// </summary>
    public string SupplierList
    {
        get => this.supplierList;
        set
        {
            if(this.supplierList == value)
            {
                return;
            }

            this.supplierList = value;

            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(SupplierList)));
        }
    }

    #endregion
    #region ǰ Ÿ - PartType

    /// <summary>
    /// ǰ Ÿ
    /// </summary>
    public string PartType
    {
        get => this.partType;
        set
        {
            if(this.partType == value)
            {
                return;
            }

            this.partType = value;

            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(nameof(PartType)));
        }
    }

    #endregion

    #region  Ϸ  - DoneEditingCommand

    /// <summary>
    ///  Ϸ 
    /// </summary>
    public ICommand DoneEditingCommand { get; private set; }

    #endregion
    #region   - SaveCommand

    /// <summary>
    ///  
    /// </summary>
    public ICommand SaveCommand { get; private set; }

    #endregion
    #region   - DeleteCommand

    /// <summary>
    ///  
    /// </summary>
    public ICommand DeleteCommand { get; private set; }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region  - AddPartViewModel()

    /// <summary>
    /// 
    /// </summary>
    public AddPartViewModel()
    {
        DoneEditingCommand = new Command(async () => await ProcessEditingDone());
        SaveCommand        = new Command(async () => await ProcessSave());
        DeleteCommand      = new Command(async () => await ProcessDelete());
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region ǰ ϱ - InsertPart()

    /// <summary>
    /// ǰ ϱ
    /// </summary>
    /// <returns>½ũ</returns>
    private async Task InsertPart()
    {
        await PartManager.AddAsync(PartName, PartType, SupplierList);

        MessagingCenter.Send(this, "refresh");

        await Shell.Current.GoToAsync("..");
    }

    #endregion
    #region ǰ ϱ - UpdatePart()

    /// <summary>
    /// ǰ ϱ
    /// </summary>
    /// <returns>½ũ</returns>
    private async Task UpdatePart()
    {
        Part partToSave = new()
        {
            PartID       = PartID,
            PartName     = PartName,
            PartType     = PartType,
            SupplierList = SupplierList.Split(",").ToList()
        };

        await PartManager.UpdateAsync(partToSave);

        MessagingCenter.Send(this, "refresh");

        await Shell.Current.GoToAsync("..");
    }

    #endregion

    #region  Ϸ óϱ - ProcessEditingDone()

    /// <summary>
    ///  Ϸ óϱ
    /// </summary>
    /// <returns>½ũ</returns>
    private async Task ProcessEditingDone()
    {
        await Shell.Current.GoToAsync("..");
    }

    #endregion
    #region  óϱ - ProcessSave()

    /// <summary>
    ///  óϱ
    /// </summary>
    /// <returns>½ũ</returns>
    private async Task ProcessSave()
    {
        if(string.IsNullOrWhiteSpace(PartID))
        {
            await InsertPart();
        }
        else
        {
            await UpdatePart();
        }
    }

    #endregion
    #region  óϱ - ProcessDelete()

    /// <summary>
    ///  óϱ
    /// </summary>
    /// <returns>½ũ</returns>
    private async Task ProcessDelete()
    {
        if(string.IsNullOrWhiteSpace(PartID))
        {
            return;
        }

        await PartManager.DeleteAsync(PartID);

        MessagingCenter.Send(this, "refresh");

        await Shell.Current.GoToAsync("..");
    }

    #endregion
}