﻿using System.Net;

using Microsoft.AspNetCore.Mvc;

using TestServer.Models;

namespace TestServer.Controllers;

/// <summary>
/// 베이스 컨트롤러
/// </summary>
[Route("api/[controller]")]
[ApiController]
public class BaseController : ControllerBase
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Property
    ////////////////////////////////////////////////////////////////////////////////////////// Protected

    #region 사용자 부품 리스트 - UserPartList

    /// <summary>
    /// 사용자 부품 리스트
    /// </summary>
    protected List<Part> UserPartList
    {
        get
        {
            if(string.IsNullOrWhiteSpace(AuthorizationToken))
            {
                return null;
            }

            if(!PartFactory.PartDictionary.ContainsKey(AuthorizationToken))
            {
                return null;
            }

            Tuple<DateTime, List<Part>> tuple = PartFactory.PartDictionary[AuthorizationToken];

            return tuple.Item2;
        }
    }

    #endregion
    #region 권한 토큰 - AuthorizationToken

    /// <summary>
    /// 권한 토큰
    /// </summary>
    protected string AuthorizationToken
    {
        get
        {
            string authorizationToken = string.Empty;

            HttpContext context = HttpContext;

            if(context != null)
            {
                authorizationToken = context.Request.Headers["Authorization"].ToString();
            }

            return authorizationToken;
        }
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Protected

    #region 권한 체크하기 - CheckAuthorization()

    /// <summary>
    /// 권한 체크하기
    /// </summary>
    /// <returns>권한 체크 결과</returns>
    protected bool CheckAuthorization()
    {
        PartFactory.ClearStaleData();

        try
        {
            HttpContext context = HttpContext;

            if(context != null)
            {
                if(string.IsNullOrWhiteSpace(AuthorizationToken))
                {
                    context.Response.StatusCode = (int)HttpStatusCode.Forbidden;

                    return false;
                }
            }
            else
            {
                return false;
            }

            if(!PartFactory.PartDictionary.ContainsKey(AuthorizationToken))
            {
                return false;
            }

            return true;
        }
        catch
        {
        }

        return false;
    }

    #endregion
}