﻿namespace TestServer.Models
{
    /// <summary>
    /// 부품 팩토리
    /// </summary>
    public static class PartFactory
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 부품 딕셔너리
        /// </summary>
        public static Dictionary<string, Tuple<DateTime, List<Part>>> PartDictionary = new Dictionary<string, Tuple<DateTime, List<Part>>>();

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 잠금자
        /// </summary>
        private static object _locker = new object();

        /// <summary>
        /// 난수 발생기
        /// </summary>
        private static readonly Random _random = new Random();

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 디폴트 부품 열거 가능형 - DefaultPartEnumerable

        /// <summary>
        /// 디폴트 부품 열거 가능형
        /// </summary>
        private static IEnumerable<Part> DefaultPartEnumerable
        {
            get
            {
                yield return new Part
                {
                    PartID            = "0545685192",
                    PartName          = "Large motherboard",
                    SupplierList      = new List<string> { "A. Datum Corporation", "Allure Bays Corp", "Awesome Computers" },
                    PartAvailableDate = new DateTime(2019, 10, 1),
                    PartType          = "Circuit Board",
                };

                yield return new Part
                {
                    PartID            = "0553801473",
                    PartName          = "RISC processor",
                    SupplierList      = new List<string> { "Allure Bays Corp", "Contoso Ltd", "Parnell Aerospace" },
                    PartAvailableDate = new DateTime(2021, 07, 12),
                    PartType          = "CPU",
                };

                yield return new Part
                {
                    PartID            = "0544272994",
                    PartName          = "CISC processor",
                    SupplierList      = new List<string> { "Fabrikam, Inc", "A. Datum Corporation", "Parnell Aerospace" },
                    PartAvailableDate = new DateTime(2020, 9, 4),
                    PartType          = "CPU",
                };

                yield return new Part
                {
                    PartID            = "141971189X",
                    PartName          = "High resolution card",
                    SupplierList      = new List<string> { "Awesome Computers" },
                    PartAvailableDate = new DateTime(2019, 11, 10),
                    PartType          = "Graphics Card",
                };

                yield return new Part
                {
                    PartID            = "1256324778",
                    PartName          = "240V/110V switchable",
                    SupplierList      = new List<string> { "Reskit" },
                    PartAvailableDate = new DateTime(2021, 10, 21),
                    PartType          = "PSU",
                };
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 초기화하기 - Initialize(authorizationToken)

        /// <summary>
        /// 초기화하기
        /// </summary>
        /// <param name="authorizationToken">권한 토큰</param>
        public static void Initialize(string authorizationToken)
        {
            lock(_locker)
            {
                PartDictionary.Add(authorizationToken, Tuple.Create(DateTime.UtcNow.AddHours(1), DefaultPartEnumerable.ToList()));
            }
        }

        #endregion
        #region 오래된 데이터 지우기 - ClearStaleData()

        /// <summary>
        /// 오래된 데이터 지우기
        /// </summary>
        public static void ClearStaleData()
        {
            lock(_locker)
            {
                List<string> keyList = PartDictionary.Keys.ToList();

                foreach(string key in keyList)
                {
                    if(PartDictionary.TryGetValue(key, out Tuple<DateTime, List<Part>> result) && result.Item1 < DateTime.UtcNow)
                    {
                        PartDictionary.Remove(key);
                    }
                }
            }
        }

        #endregion
        #region 부품 ID 생성하기 - CreatePartID()

        /// <summary>
        /// 부품 ID 생성하기
        /// </summary>
        /// <returns>부품 ID</returns>
        public static string CreatePartID()
        {
            char[] characterArray = new char[10];

            for(int i = 0; i < 10; i++)
            {
                characterArray[i] = (char)('0' + _random.Next(0, 9));
            }

            return new string(characterArray);
        }

        #endregion
    }
}